# rspec/airborne/spaceborne based API tests

## Setup
If you have rvm set up reasonably on your system, the .ruby-XXX files in the apitest directory should create the rvm gemset for you. You will then need to:

```ruby
gem install bundler
bundle install
```

## Environment
| Environment Variable | values | default |
|:------ |:----- |:----- |
| TEST_ENV | dev, local, qa, staging, prod | dev |
| NAMESPACE | as in other test environments | |                    
| API_KEY   | api key for redshirt account | |
| REDSHIRT_PW | the redshirt account password | |
| EKS_ENV | control the context | virginia-1 |

to invoke your service tests tests (from this directory)

```ruby
TEST_ENV=staging EKS_ENV=virginia-1 rspec -f d
```

to only run one spec file

```ruby
TEST_ENV=staging EKS_ENV=virginia-1 rspec -f d spec/YOUR_FEATURE_spec.rb
```

to only run part of one spec - append a ':123' where 123 is the line number in the describe or it block

to have a failing test perform and output the response using curl, change the appropriate header to dbg_header or set the environment variable ```FORCE_CURL_DEBUG=true```

## Failures
### Schema changes
In the event of a schema change, you will get a test failure of the test like:

QueryEngineFlushing introspection - graphql schema changes for query-engine-flushing detects schema changes for query-engine-flushing

First - Alert the QE or Team member responsible for the API tests, and make sure they are aware of the schema change, and create stories to enhance the tests to fix the discrepency between the apitests in the branch/repo, and the schema changes that are part of the PR. This usually involves adding some test cases or validation for new fields or calls.

Second - You can update the supported schema on your branch so that you will no longer see the failure. This would usually be done against the pods running your service for the jenkins PR build that failed the schema change test. From your local system, in the service's apitest directory, run the following - changing the namespace to the one used for the jenkins run

```TEST_ENV=staging EKS_ENV=virginia-1 NAMESPACE=query-engine-flushing-pr-### bundle exec rspec --tag save_schema```

This will change the schema in the supported_schemas directory on your branch, which you can then add to your PR to get the tests to pass.

The tests are not there to make your life harder, but to ensure that we don't have schema changes being deployed without QE being aware that test changes are required.

## Instrument for hawkeye apicheck monitoring

required ENV variables

```ruby
  export HAWKEYE_USER="hawkeyeuser"
  export HAWKEYE_PASSWORD="..." # from the production infrastructure page
```

First step will be to ensure that your apitests run with TEST_ENV=staging and EKS_ENV=external. This causes all api requests to be done via api-proxy, so you might have to defeat some tests from running, usually with rspec tags, and preventing them from running with EKS_ENV==external.
Next step is to pick a test that will not change over the lifetime of your service (such as having a get that will return more records tomorrow than it does today). A good pick is usually a node call if your service supports that.
Then you would instrument a positive test case that runs fairly quickly, and add an extra argument to the graphql call in that test case. An example follows:

```ruby
                  hawkeye: {
                    name: 'chapig_organization_node', # chapig stands for cloudhealth api generated
		    	  			      # organization is the service name
						      # node is the call 
                    service: service,		      # service points at the service under test
                    output: [{ type: 'kpi',
                               path: ['id'],	      # the path and name are fields that should exist in the response
                               name: 'id' }]
                  }
```
Next, enable GlobalProtect, disable Pritunl, and run the following

```ruby
HAWKEYE_CONTROL=XXX TEST_ENV=staging EKS_ENV=external rspec spec/YOUR_API_TEST_SPEC:line_number
```

where XXX is one of:

```ruby
query - tells you if the apicheck doesn't exist in hawkeye, or diffs against the existing one
debug - gives you the info that would be written into an apicheck
upsert - actually inserts / updates the apicheck in hawkeye
```

The apicheck will still not be automatically running. Use the swagger page for hawkeye to enable it in staging and/or production

