# frozen_string_literal: true

require 'cht-chaphql'
require 'cht-chaphql/graphql_formatters'

QUERY_ENGINE_FLUSHING_FMTRS =
  YAML.load_file(File.join(__dir__, '../helpers/formatters.yml'))

# module for query-engine-flushing
module ChtApitestQueryEngineFlushing
  include Chaphql
  include Chaphql::GraphQLHelpers

  # module for your shortcuts
  module QueryEngineFlushingHelpers
    def via_public
      %w[staging canary prod].include?(getenv)
    end

    def auth_service
      LocalAuthService.new(via_public: via_public)
    end
    # insertion point for shortcuts
    # end of shortcut insertion
  end

  # class for your service
  class QueryEngineFlushingService < ChtNgService
    include RSpec::Matchers

    QUERY_ENGINE_FLUSHING_DEFAULTS = {
      name: 'QueryEngineFlushing',
      default_namespace: 'query-engine-flushing',
      service_name: 'query-engine-flushing',
      pr_parent: 'query-engine-flushing',
      formatters: QUERY_ENGINE_FLUSHING_FMTRS['formatters'],
      v2_checkers: QUERY_ENGINE_FLUSHING_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(QUERY_ENGINE_FLUSHING_DEFAULTS))
    end
  end

  # class for api-proxy support of query-engine-flushing service
  class ApiProxyQueryEngineFlushing < ChtNgService
    include RSpec::Matchers

    API_PROXY_DEFAULTS = {
      name: 'ApiProxyQueryEngineFlushing',
      default_namespace: 'api-proxy',
      service_name: 'api-proxy',
      pr_parent: 'api-proxy',
      formatters: QUERY_ENGINE_FLUSHING_FMTRS['formatters'],
      v2_checkers: QUERY_ENGINE_FLUSHING_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(API_PROXY_DEFAULTS))
    end
  end
end
